//
//  AsyncClientAppDelegate.m
//  AsyncClient
//
//  Simple sample client to demonstrate AsyncSocket usage.
//  http://code.google.com/p/cocoaasyncsocket/
//  Try with EchoServer, AsyncSocket's example server
//
//  Created by Dirk Fröhling on 26.05.11.
//  Copyright 2011 FH Gelsenkirchen. All rights reserved.
//

#import "AsyncClientAppDelegate.h"
#import "AsyncSocket.h"

#define MSG_TAG  0
#define READ_TIMEOUT 45.0

@implementation AsyncClientAppDelegate

@synthesize window;

#pragma mark Initialisation

- (id)init
{
	if((self = [super init]))
	{
		socket = [[AsyncSocket alloc] initWithDelegate:self];
		
		isConnected = NO;
	}
	return self;
}


- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    [sendButton setEnabled:NO];
    [messageField setEnabled:NO];
    [antwortField setEnabled:NO];
	// Advanced options - enable the socket to contine operations even during modal dialogs, and menu browsing
	[socket setRunLoopModes:[NSArray arrayWithObject:NSRunLoopCommonModes]];
}

#pragma mark Action Methods

- (IBAction)connect:(id)sender {
    if (!isConnected) {
        int port = [portField intValue];
        NSString *host = [hostField stringValue];
        
        if(port < 0 || port > 65535)
        {
            port = 0;
        }
        NSLog(@"Connect with host %@, Port %i",host, port);
        [socket connectToHost:host onPort:port error:nil];
        // Handle success in callback
    }
    else {
		[socket disconnect];
        // Handle success in callback
    }
}


- (IBAction)sendMessage:(id)sender {
    // Turn texfield content into message
	NSMutableString *message = [[NSMutableString alloc] init];
    [message setString:[messageField stringValue]];
	[message appendString:@"\r\n"];
	NSData *nachrichtData = [message dataUsingEncoding:NSUTF8StringEncoding];
	// Send message
	[socket writeData:nachrichtData withTimeout:-1 tag:MSG_TAG];
}

#pragma mark Callback Methods

- (void)onSocket:(AsyncSocket *)sock didConnectToHost:(NSString *)host port:(UInt16)port {
    NSLog(@"Connected.");
    // Adjust GUI
    [connectButton setTitle:@"Disconnect"];
    [sendButton setEnabled:YES];
    [sendButton setKeyEquivalent:@"\r"];
    [connectButton setKeyEquivalent:@""];
    [messageField setEnabled:YES];
    [antwortField setEnabled:YES];
    [hostField setEnabled:NO];
    [portField setEnabled:NO];
    isConnected = TRUE;
    // Async wait for data. Tag doesn't get to the other side,
    // its is just for possible sorting purposes
	[sock readDataToData:[AsyncSocket CRLFData] withTimeout:READ_TIMEOUT tag:MSG_TAG];
}


-(void)onSocketDidDisconnect:(AsyncSocket *)sock {
    NSLog(@"Connection split.");
    // Adjust GUI
    [connectButton setTitle:@"Connect"];
    [sendButton setEnabled:NO];
    [connectButton setKeyEquivalent:@"\r"];
    [sendButton setKeyEquivalent:@""];
    [messageField setEnabled:NO];
    [antwortField setEnabled:NO];
    [hostField setEnabled:YES];
    [portField setEnabled:YES];
    isConnected = FALSE;        
}


- (void)onSocket:(AsyncSocket *)sock didReadData:(NSData *)data withTag:(long)tag {
    // Strip CR LF
	NSData *strData = [data subdataWithRange:NSMakeRange(0, [data length] - 2)];
    // Convert data into string
	NSString *msg = [[[NSString alloc] initWithData:strData encoding:NSUTF8StringEncoding] autorelease];
	if(msg) {
        NSLog(@"Got message: %@", msg);        
        if (tag != MSG_TAG) // We only want messages of type MSG_TAG
            NSLog(@"Tag: %li", tag);
        [antwortField setStringValue:msg];
	}
	else {
        NSLog(@"Data not readable!");
        [antwortField setStringValue:@"Data not readable!"];
	}
    // Prepare for next message. Tag will not get sent, only for our sorting purposes
    [sock readDataToData:[AsyncSocket CRLFData] withTimeout:READ_TIMEOUT tag:MSG_TAG];
}


- (void)onSocket:(AsyncSocket *)sock didWriteDataWithTag:(long)tag {
    NSLog(@"Sent message.");
}


@end
